import os
from django.db import models
import traceback
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.db.models import signals
from django.conf import settings

class Element(models.Model):
    """
    The Element model will contain images and videos
    NOTE: if type=video you can still have a thumbnail_image
    """

    TYPE_CHOICES = (('image','Image'),('video','Video'))

    name = models.CharField(max_length=150,blank=True,null=True)
    file_name = models.CharField(max_length=150,blank=True,null=True)
    original_file_name = models.CharField(max_length=150,blank=True,null=True)
    credit = models.CharField(max_length=255,blank=True,null=True)
    description = models.TextField(blank=True,null=True)
    image = models.ImageField(blank=True,null=True,max_length=255,upload_to="images/")
    image_url = models.CharField(max_length=255,blank=True,null=True)
    image_width = models.IntegerField(blank=True,null=True,default='0')
    image_height = models.IntegerField(blank=True,null=True,default='0')
    video_url = models.CharField(max_length=255,blank=True,null=True)
    video_embed = models.TextField(blank=True,null=True)
    thumbnail_image = models.ImageField(blank=True,null=True,max_length=255,upload_to="images/")
    thumbnail_image_url = models.CharField(max_length=255,blank=True,null=True)
    thumbnail_image_width = models.IntegerField(blank=True,null=True,default='0')
    thumbnail_image_height = models.IntegerField(blank=True,null=True,default='0')
    type = models.CharField(_("Type"), max_length=10, default="image",choices=TYPE_CHOICES)
    created_at = models.DateTimeField(blank=True,null=True,auto_now_add=True)
    updated_at = models.DateTimeField(blank=True,null=True,auto_now=True)

    class Meta:
        verbose_name_plural = "Elements"

    def __unicode__(self):
        return u"%s" % (self.name)

    def save(self, *args, **kwargs):
        if not self.name:
            if self.type == "image":
                self.name = self.file_name
            elif self.type == "video":
                self.name = self.video_url
        super(Element, self).save(*args, **kwargs)

    def save(self, *args, **kwargs):

        if not self.name:
            if self.type == "image":
                self.name = self.file_name
            elif self.type == "video":
                self.name = self.video_url

        super(Element, self).save(*args, **kwargs)

        save_again = False

        #Init thumbnail_image
        if self.type == "image" \
                and self.image \
                and not self.thumbnail_image:
            self.image_url = self.image.url
            self.thumbnail_image = self.image
            self.thumbnail_image_url = self.image.url
            save_again = True

        if save_again:
            super(Element, self).save(*args, **kwargs)

class Gallery(models.Model):
    """
    The Gallery model will contain info about our media gallery
    """

    name = models.CharField(max_length=100,blank=True,null=True)
    short_code = models.CharField(max_length=100,blank=True,null=True)
    description = models.TextField(blank=True,null=True)
    thumbnail_image = models.ImageField(blank=True,null=True,max_length=255,upload_to="images/")
    thumbnail_image_url = models.CharField(max_length=255,blank=True,null=True)
    elements = models.ManyToManyField(Element, through="GalleryElement")
    created_at = models.DateTimeField(blank=True,null=True,auto_now_add=True)
    updated_at = models.DateTimeField(blank=True,null=True,auto_now=True)

    class Meta:
        verbose_name_plural = "Galleries"

    def __unicode__(self):
        return u"%s" % (self.name)

class GalleryElement(models.Model):
    """
    The Gallery Element model will contain list of elements
    """

    gallery = models.ForeignKey(Gallery)
    element = models.ForeignKey(Element)
    credit = models.CharField(max_length=255,blank=True,null=True)
    description = models.TextField(blank=True,null=True)
    sort_by = models.IntegerField(blank=True,null=True,default='0')
    created_at = models.DateTimeField(blank=True,null=True,auto_now_add=True)
    updated_at = models.DateTimeField(blank=True,null=True,auto_now=True)

    class Meta:
        verbose_name = "Gallery element"
        verbose_name_plural = "Gallery elements"
        ordering = ["sort_by"]

class ResizedImage(models.Model):
    """
    The ResizedImage is a resized image version of Element.image
    """

    image = models.ForeignKey(Element)
    file_name = models.CharField(max_length=150,blank=True,null=True)
    size = models.CharField(max_length=25,blank=True,null=True)
    image_url = models.CharField(max_length=255,blank=True,null=True)
    image_width = models.IntegerField(blank=True,null=True,default='0')
    image_height = models.IntegerField(blank=True,null=True,default='0')
    created_at = models.DateTimeField(blank=True,null=True,auto_now_add=True)
    updated_at = models.DateTimeField(blank=True,null=True,auto_now=True)

    class Meta:
        verbose_name = "Resized image"
        verbose_name_plural = "Resized images"

    def html_img(self):
        try:
            return "<a target='_blank' href='" + self.image_url + "'><img style='width:100px' src='" + self.image_url + "'></a>"
        except:
            pass

    html_img.allow_tags = True

def resizedimage_post_delete(sender, instance, **kwargs):
    """
    Deletes file from filesystem when corresponding `ResizedImage` object is deleted.
    """

    try:
        if instance.image_url:
            if os.path.isfile(settings.PROJECT_ROOT + instance.image_url):
                os.remove(settings.PROJECT_ROOT + instance.image_url)
    except:
        print traceback.format_exc()

def element_post_delete(sender, instance, **kwargs):
    """
    Deletes file from filesystem when corresponding `Element` object is deleted.
    """

    try:
        if instance.image:
            if os.path.isfile(instance.image.path):
                os.remove(instance.image.path)
    except:
        print traceback.format_exc()

    try:
        if instance.thumbnail_image:
            if os.path.isfile(instance.thumbnail_image.path):
                os.remove(instance.thumbnail_image.path)
    except:
        print traceback.format_exc()

    try:
        if instance.thumbnail_image_url:
            if os.path.isfile(settings.PROJECT_ROOT + instance.thumbnail_image_url):
                os.remove(settings.PROJECT_ROOT + instance.thumbnail_image_url)
    except:
        print traceback.format_exc()

    try:
        #Check to see if this thumbanil is used by gallery - if so resave the gallery to reset thumbnail
        for gallery in Gallery.objects.filter(thumbnail_image_url=instance.thumbnail_image_url):
            gallery.save()
    except:
        print traceback.format_exc()

def gallery_post_save(sender, instance, created, **kwargs):
    #Disconnect signal here so we don't recurse when we save
    signals.post_save.disconnect(gallery_post_save, sender=Gallery)

    #Grap the thumbnail URL from the first element - or use the default
    has_thumbnail = False
    try:
        #Get the first element and set the thumbnail
        if GalleryElement.objects.filter(gallery=instance).exists():
            ge = GalleryElement.objects.filter(gallery=instance).order_by("sort_by")[0]
            if ge.element.thumbnail_image_url:
                has_thumbnail = True
                instance.thumbnail_image_url = ge.element.thumbnail_image_url
                instance.save()
    except:
        print traceback.format_exc()

    if not has_thumbnail:
        instance.thumbnail_image_url = settings.DME_GALLERY_THUMBNAIL_DEFAULT_URL
        instance.save()

    #Reconnect signal
    signals.post_save.connect(gallery_post_save, sender=Gallery)


def element_post_save(sender, instance, created, **kwargs):

    #Disconnect signal here so we don't recurse when we save
    signals.post_save.disconnect(element_post_save, sender=Element)

    if instance.video_url or instance.video_embed:
        instance.type = "video"

    if instance.image:
        instance.image_url = instance.image.url
      	instance.file_name = os.path.basename(str(instance.image_url))
      	if not instance.name:
            instance.name = instance.file_name

        instance.thumbnail_image = instance.image

    if instance.thumbnail_image:
        instance.thumbnail_image_url = instance.thumbnail_image.url

    instance.save()

    if instance.video_url:
        try:
            import micawber
            providers = micawber.bootstrap_basic()
            oembed = providers.request(instance.video_url)
            if "html" in oembed:
                instance.video_embed = oembed["html"]

                if not instance.thumbnail_image:
                    if "thumbnail_url" in oembed:
                        instance.thumbnail_image_url = oembed["thumbnail_url"]
                    if "thumbnail_width" in oembed:
                        instance.thumbnail_image_height = oembed["thumbnail_width"]
                    if "thumbnail_height" in oembed:
                        instance.thumbnail_image_height = oembed["thumbnail_height"]
                
        except Exception as e:
            print traceback.format_exc()

    #Process images and thumbnails
    try:
        if instance.image and instance.file_name != instance.original_file_name:
            instance.original_file_name = instance.file_name
            instance.save()
            from .helpers import ImageHelper
            helper = ImageHelper()
            rtn = helper.resize(instance)
            if rtn["success"]: 
                if rtn["thumbnail_image_url"]:
                    instance.thumbnail_image = ""
                    instance.thumbnail_image_url = rtn["thumbnail_image_url"]
                    instance.save()
            else:
                print rtn["message"]
    except Exception as e:
        print traceback.format_exc()

    #If there is still no thumbnail image then use the default
    if instance.type == "video" and not instance.thumbnail_image_url:
        instance.thumbnail_image_url = settings.DME_VIDEO_THUMBNAIL_DEFAULT_URL

    instance.save()

    #Reconnect signal
    signals.post_save.connect(element_post_save, sender=Element)

signals.post_save.connect(element_post_save, sender=Element)
signals.post_save.connect(gallery_post_save, sender=Gallery)
signals.post_delete.connect(element_post_delete, sender=Element)
signals.post_delete.connect(resizedimage_post_delete, sender=ResizedImage)



from __future__ import unicode_literals
import os
from django.test import TestCase, Client, override_settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from media_explorer.models import Element, ResizedImage

class ElementTests(TestCase):

    def setUp(self):
        #Create test users
        User.objects.create_user("testuser",password="password")
        User.objects.create_superuser("admin","admin@example.com","password")

    def test_element_post_while_user_is_logged_out(self):
        """
        Test post while user is logged out
        Condition: User is logged out
        Result: Fail with 403 error since user is not authenticated
        """
        url = reverse("api-media-elements")
        c = Client()
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)

    def test_element_post_while_user_is_not_authorized(self):
        """
        Test post while user is not authorized
        Condition: User is logged in but not authorized
        Result: Fail with 403 error since user is not staff
        """
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="testuser",password="password")
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)


    def test_element_post_with_no_image_or_video(self):
        """
        Test post with no image file or video url
        Condition: User is logged in and authorized
        Condition: Post has no image file or video url
        Result: Fail with 400 error since no image or video_url is provided
        """
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="admin",password="password")
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 400)

    @override_settings(DME_RESIZE=False)
    def test_image_upload_with_no_resize(self):
        """
        Test image upload with no resize
        Condition: User is logged in and authorized
        Condition: Post has an image
        Condition: Do not resize image
        Result: Success
        Result: Element count should equal 1
        Result: ResizedImage count should equal 0
        """
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="admin",password="password")

        test_path = os.path.dirname(os.path.abspath(__file__))
        with open(test_path + '/Oxfam-Cambodia.jpg') as fp:
            response = c.post(url, {'name':'test_image_upload_with_no_resize','image':fp},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        #Check the DB to make sure element is present
        count1 = Element.objects.filter(type="image",name="test_image_upload_with_no_resize",image_url__icontains="Oxfam-Cambodia").count()
        self.assertEqual(count1, 1)

        #Check to make sure image was NOT resized
        count2 = ResizedImage.objects.filter(image__type="image",image__name="test_image_upload_with_no_resize",image__image_url__icontains="Oxfam-Cambodia").count()
        self.assertEqual(count2, 0)

    @override_settings(DME_RESIZE=True)
    def test_image_upload_with_resize(self):
        """
        Test image upload with resize
        Condition: User is logged in and authorized
        Condition: Post has an image
        Condition: Resize image
        Result: Success
        Result: Element count should equal 1
        Result: ResizedImage count should be more than 0
        """
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="admin",password="password")

        test_path = os.path.dirname(os.path.abspath(__file__))
        with open(test_path + '/Oxfam-Cambodia.jpg') as fp:
            response = c.post(url, {'name':'test_image_upload_with_resize','image':fp},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        #Check the DB to make sure element is present
        count1 = Element.objects.filter(type="image",name="test_image_upload_with_resize",image_url__icontains="Oxfam-Cambodia").count()
        self.assertEqual(count1, 1)

        #Check to make sure image was NOT resized
        count2 = ResizedImage.objects.filter(image__type="image",image__name="test_image_upload_with_resize",image__image_url__icontains="Oxfam-Cambodia").count()
        self.assertTrue(count2>0)


    def test_youtube_video_addition(self):
        """
        Test Youtube video addition
        Condition: User is logged in and authorized
        Condition: Post has a video url
        Result: Success
        Result: Element count (with non-empty video_embed) should equal 1
        """
        url = reverse("api-media-elements")

        c = Client()
        c.login(username="admin",password="password")

        response = c.post(url, {'name':'test_youtube_video_addition','video_url':'https://www.youtube.com/watch?v=nBupSKjSQM0'},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        #Check the DB to make sure element is present
        count = Element.objects.filter(type="video",name="test_youtube_video_addition",video_embed__isnull=False).count()
        self.assertEqual(count, 1)



import math, json
from django.http import HttpResponse
from django.views.generic import View
from media_explorer.models import Element, Gallery, GalleryElement
from django.conf import settings
from django.db.models import Q

class ElementStatsView(View):

    def get(self, request, *args, **kwargs):
        #NOTE: Code below must match ElementList.get_queryset
        #TODO - move to helper so you don't maintain same code twice
        data = {}
        query = None
        and_query = None
        or_query = None
        type = request.GET.get('type', None)
        filter = request.GET.get('filter', None)

        data["page_size"] = settings.DME_PAGE_SIZE

        try:
            if type:
                and_query = Q(type=type)
        except Exception as e:
            pass

        if filter:
            try:
                filter_list = filter.split(" ")
                or_query = Q(name__icontains=filter_list[0]) | Q(description__icontains=filter_list[0]) | Q(credit__icontains=filter_list[0])
                for term in filter_list[1:]:
                    or_query.add((Q(name__icontains=term) | Q(description__icontains=term) | Q(credit__icontains=term)), or_query.connector)
            except Exception as e:
                pass

        if and_query:
            query = and_query

        if or_query:
            if query:
                query.add(or_query,Q.AND)
            else:
                query = or_query

        if query:
            data["total_entries"] = Element.objects.filter(query).count()
        else:
            data["total_entries"] = Element.objects.all().count()

        data["total_pages"] = int(math.ceil(float(data["total_entries"])/data["page_size"]))

        return HttpResponse(json.dumps(data),content_type="application/json")

class GalleryStatsView(View):
    def get(self, request, *args, **kwargs):
        #NOTE: Code below must match GalleryList.get_queryset
        #TODO - move to helper so you don't maintain same code twice
        data = {}
        query = None
        and_query = None
        or_query = None
        filter = request.GET.get('filter', None)

        data["page_size"] = settings.DME_PAGE_SIZE

        if filter:
            try:
                filter_list = filter.split(" ")
                or_query = Q(name__icontains=filter_list[0]) | Q(description__icontains=filter_list[0])
                for term in filter_list[1:]:
                    or_query.add((Q(name__icontains=term) | Q(description__icontains=term)), or_query.connector)
            except Exception as e:
                pass

        if and_query:
            query = and_query

        if or_query:
            if query:
                query.add(or_query,Q.AND)
            else:
                query = or_query

        if query:
            data["total_entries"] = Gallery.objects.filter(query).count()
        else:
            data["total_entries"] = Gallery.objects.all().count()

        data["total_pages"] = int(math.ceil(float(data["total_entries"])/data["page_size"]))

        return HttpResponse(json.dumps(data),content_type="application/json")




from __future__ import unicode_literals
import os
from django.test import TestCase, Client, override_settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from media_explorer.models import Element, Gallery, GalleryElement

class GalleryTests(TestCase):

    def setUp(self):
        #Create test users
        User.objects.create_user("testuser",password="password")
        User.objects.create_superuser("admin","admin@example.com","password")

    def test_gallery_post_while_user_is_logged_out(self):
        """
        Test post while user is logged out
        Condition: User is logged out
        Result: Fail with 403 error since user is not authenticated
        """
        url = reverse("api-media-galleries")
        c = Client()
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)

    def test_gallery_post_while_user_is_not_authorized(self):
        """
        Test post while user is not authorized
        Condition: User is logged in but not authorized
        Result: Fail with 403 error since user is not staff
        """
        url = reverse("api-media-galleries")
        c = Client()
        c.login(username="testuser",password="password")
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 403)


    def test_gallery_post_with_no_data(self):
        """
        Test post with data
        Condition: User is logged in and authorized
        Condition: Post has no data
        Result: Fail with 400 error since at least name is required
        """
        url = reverse("api-media-galleries")
        c = Client()
        c.login(username="admin",password="password")
        response = c.post(url, {},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 400)

    def test_gallery_post_with_at_least_name_provided(self):
        """
        Test post with data
        Condition: User is logged in and authorized
        Condition: Post has only name field
        Result: Success
        """
        url = reverse("api-media-galleries")
        c = Client()
        c.login(username="admin",password="password")
        response = c.post(
            url, 
            {'name':'test_post_with_at_least_name'},
            HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

    @override_settings(DME_RESIZE=False)
    def test_add_image_with_credit_and_video_during_gallery_creation(self):
        """
        Test image upload with no resize
        Condition: User is logged in and authorized
        Condition: Post has an image
        Condition: Do not resize image
        Result: Success
        Result: GalleryElement count should be equal 2
        Result: GalleryElement is associated with correct gallery, image and video elements
        Result: GalleryElement image has the correct credit
        """
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="admin",password="password")

        test_path = os.path.dirname(os.path.abspath(__file__))

        #Add an image element
        with open(test_path + '/../elements/Oxfam-Cambodia.jpg') as fp:
            response = c.post(url, {'name':'test_add_image_with_credit_and_video_during_gallery_creation-image','image':fp},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        image = Element.objects.get(type="image",name="test_add_image_with_credit_and_video_during_gallery_creation-image",image_url__icontains="Oxfam-Cambodia")

        #Add a video element
        response = c.post(url, {'name':'test_add_image_with_credit_and_video_during_gallery_creation-video','video_url':'https://www.youtube.com/watch?v=nBupSKjSQM0'},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        video = Element.objects.get(type="video",name="test_add_image_with_credit_and_video_during_gallery_creation-video",video_embed__isnull=False)

        #Add a gallery with the image and video created 
        url = reverse("api-media-galleries")
        params = {}
        params["name"] = "test_add_image_with_credit_and_video_during_gallery_creation-gallery"
        params["element_id"] = [image.id,video.id]
        #Note element_credit, element_description should be in same order as element_id
        params["element_credit"] = ["Image by Oxfam"]
        #Note we are not even going to provide a list of element_description
        #params["element_description"] = []

        response = c.post(url, params,HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        count1 = Gallery.objects.filter(name="test_add_image_with_credit_and_video_during_gallery_creation-gallery").count()
        self.assertEqual(count1, 1)

        gallery = Gallery.objects.get(name="test_add_image_with_credit_and_video_during_gallery_creation-gallery")

        count2 = GalleryElement.objects.count()
        self.assertEqual(count2, 2)

        for ge in GalleryElement.objects.all():
            self.assertEqual(ge.gallery, gallery)
            if ge.element.id == image.id:
                self.assertEqual(ge.element, image)
                self.assertEqual(ge.credit, "Image by Oxfam")
            if ge.element.id == video.id:
                self.assertEqual(ge.element, video)






import os, re
from django.conf import settings
from django.db.models import get_model

try:
	from PIL import Image, ImageOps
except ImportError:
	import Image
	import ImageOps

class ImageHelper(object):

    def resize(self, instance):
        rtn = {}
        rtn["success"] = False
        rtn["message"] = ""
        rtn["thumbnail_image_url"] = None

        url = instance.image_url
        full_path = settings.PROJECT_ROOT + "/" + url.strip("/")

        try:
            if os.path.exists(full_path):
                image = Image.open(full_path)
            else:
                rtn["message"] = "File path does not exist"
                return rtn
        except Exception as e:
            rtn["message"] = e.__str__()
            return rtn

        image_width, image_height = image.size
        instance.image_width = image_width
        instance.image_height = image_height
        instance.save()

        if not settings.DME_RESIZE:
            rtn["message"] = "The image was not resized since settings.DME_RESIZE is set to False"
            return rtn

        #create DME_RESIZE_DIRECTORY directory
        resize_dir = settings.PROJECT_ROOT + "/"
        resize_dir += settings.MEDIA_URL.strip("/") +  "/"
        resize_dir += settings.DME_RESIZE_DIRECTORY.strip("/")
        if not os.path.exists(resize_dir): os.makedirs(resize_dir)
        new_dir = "/" + settings.MEDIA_URL.strip("/") +  "/" + settings.DME_RESIZE_DIRECTORY.strip("/") + "/"

        #Clean file name
        file_name = os.path.basename(url)
        file_name = re.sub(r'\'|\"|\(|\)|\$','',file_name)
        file_name = re.sub(r'\s','-',file_name)

        extension = ""

        if "." in file_name:
            file_name_array = file_name.split(".")
            extension = file_name_array.pop()
            file_name = ".".join(file_name_array)

        if extension.lower() not in ["png","jpg","gif","bmp","jpeg","tiff"]:
            extension = image.format.lower()

        extension = extension.lower()

        url_orig_cropped = new_dir + file_name + "_orig_c." + extension
        orig_cropped_height = image_height
        orig_cropped_width = image_width

        #Aspect ratio for horizontal numerator
        ar_h_n = int(settings.DME_RESIZE_HORIZONTAL_ASPECT_RATIO.split(":")[0])
        #Aspect ratio for horizontal denominator
        ar_h_d = int(settings.DME_RESIZE_HORIZONTAL_ASPECT_RATIO.split(":")[1])
        #Aspect ratio for vertical numerator
        ar_v_n = int(settings.DME_RESIZE_VERTICAL_ASPECT_RATIO.split(":")[0])
        #Aspect ratio for vertical denominator
        ar_v_d = int(settings.DME_RESIZE_VERTICAL_ASPECT_RATIO.split(":")[1])

        if image_width > image_height:
            #Horizontal
            #orig_cropped_height = int(5/float(8)*image_width)
            orig_cropped_height = int(ar_h_d/float(ar_h_n)*image_width)

            if orig_cropped_height > image_height:
                orig_cropped_height = image_height
                #orig_cropped_width = int(8/float(5)*image_height)
                orig_cropped_width = int(ar_h_n/float(ar_h_d)*image_height)

        elif image_width < image_height:
            #Vertical
            #orig_cropped_width = int(320/float(414)*image_height)
            orig_cropped_width = int(ar_v_n/float(ar_v_d)*image_height)
            if orig_cropped_width > image_width:
                orig_cropped_width = image_width
                #orig_cropped_height = int(414/float(320)*image_width)
                orig_cropped_height = int(ar_v_d/float(ar_v_n)*image_width)
        else:
            #Square
            pass

        rtn_crop = self._crop_and_resize(image, orig_cropped_width, orig_cropped_height, url_orig_cropped)
        if not rtn_crop["success"]:
            return rtn_crop

        ResizedImage = get_model("media_explorer","ResizedImage")

        #We will work from the aspect-ratio cropped out version
        ri = ResizedImage()
        ri.image = instance
        ri.file_name = file_name + "_orig_c." + extension
        ri.image_url = url_orig_cropped
        ri.image_height = orig_cropped_height
        ri.image_width = orig_cropped_width
        ri.size = "orig_c"
        ri.save()

        image_cropped = Image.open(settings.PROJECT_ROOT + url_orig_cropped)

        ar_d = 0
        ar_n = 0
        if orig_cropped_width > orig_cropped_height:
            image_orientation = "horizontal"
            ar_d = ar_h_d
            ar_n = ar_h_n
        elif orig_cropped_width < orig_cropped_height:
            image_orientation = "vertical"
            ar_d = ar_v_d
            ar_n = ar_v_n
        else:
            image_orientation = "square"

		#Handle horizontal and vertical images
        if image_orientation in ["horizontal","vertical"]:
            for size_width in settings.DME_RESIZE_WIDTHS[image_orientation]:

                size_height = int(size_width*ar_d/ar_n)
                size = str(size_width) + "x" + str(size_height)

                if (orig_cropped_width >= size_width) and (orig_cropped_height >= size_height):
                    new_file_name = file_name + "_" + size + "." + extension
                    new_url = new_dir + new_file_name
                    rtn_resize = self._crop_and_resize(image_cropped, size_width, size_height, new_url)
                    if rtn_resize["success"]:
                        ri = ResizedImage()
                        ri.image = instance
                        ri.file_name = new_file_name
                        ri.image_url = new_url
                        ri.image_height = size_height
                        ri.image_width = size_width
                        ri.size = size
                        ri.save()

                    if size_width in settings.DME_RESIZE_WIDTHS["retina_2x"]:
                        retina_size_width = 2*size_width
                        retina_size_height = int(retina_size_width*ar_d/ar_n)
                        retina_size = str(retina_size_width) + "x" + str(retina_size_height)

                        if (orig_cropped_width >= retina_size_width) and (orig_cropped_height >= retina_size_height):
                            new_file_name = file_name + "_" + size + "@2x." + extension
                            new_url = new_dir + new_file_name
                            rtn_resize = self._crop_and_resize(image_cropped, retina_size_width, retina_size_height, new_url)
                            if rtn_resize["success"]:
                                ri = ResizedImage()
                                ri.image = instance
                                ri.file_name = new_file_name
                                ri.image_url = new_url
                                ri.image_height = retina_size_height
                                ri.image_width = retina_size_width
                                ri.size = size + "@2x"
                                ri.save()

		#Handle non-cropped images (vertical, horizontal, square)
        for size_width in settings.DME_RESIZE_WIDTHS["non_cropped"]:
            size_height = int(image_height/float(image_width)*size_width)
            size = str(size_width) + "nc"

            retina_size_width = 2*size_width
            retina_size_height = int(image_height/float(image_width)*retina_size_width)

            if (image_width >= size_width) and (image_height >= size_height):
                new_file_name = file_name + "_" + size + "." + extension
                new_url = new_dir + new_file_name
                rtn_resize = self._crop_and_resize(image, size_width, size_height, new_url)
                if rtn_resize["success"]:
                    ri = ResizedImage()
                    ri.image = instance
                    ri.file_name = new_file_name
                    ri.image_url = new_url
                    ri.image_height = size_height
                    ri.image_width = size_width
                    ri.size = size
                    ri.save()


            if (image_width >= retina_size_width) and (image_height >= retina_size_height):
                new_file_name = file_name + "_" + size + "@2x." + extension
                new_url = new_dir + new_file_name
                rtn_resize = self._crop_and_resize(image, retina_size_width, retina_size_height, new_url)
                if rtn_resize["success"]:
                    ri.image = instance
                    ri.file_name = new_file_name
                    ri.image_url = new_url
                    ri.image_height = retina_size_height
                    ri.image_width = retina_size_width
                    ri.size = size + "@2x"
                    ri.save()

        #Now process thumbnail_image_url
        size_width = settings.DME_RESIZE_WIDTHS["thumbnail"]
        size_height = int(image_height/float(image_width)*size_width)
        size = str(size_width) + "x" + str(size_height) + ".thumbnail"

        new_file_name = file_name + "_" + size + "." + extension
        new_url = new_dir + new_file_name
        rtn_resize = self._crop_and_resize(image, size_width, size_height, new_url)
        if rtn_resize["success"]:
            rtn["thumbnail_image_url"] = new_url

        rtn["success"] = True
        return rtn


    def _crop_and_resize(self, image, width, height, new_path):
        rtn = {}
        rtn["success"] = False
        rtn["message"] = ""
        try:
            imagefit = ImageOps.fit(image, (width, height), Image.ANTIALIAS)
            imagefit.save(settings.PROJECT_ROOT + new_path, image.format, quality=100)
        except Exception as e:
            rtn["message"] = e.__str__()
            return rtn

        rtn["success"] = True
        return rtn

    def _resize(self, image, width, height, new_path):
        rtn = {}
        rtn["success"] = False
        rtn["message"] = ""
        try:
            resized = image.resize((width,height),Image.ANTIALIAS).save(settings.PROJECT_ROOT + new_path)
        except Exception as e:
            rtn["message"] = e.__str__()
            return rtn

        rtn["success"] = True
        return rtn

#EOF



import json
from django.db import models
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _
from media_explorer.models import Element, Gallery
from media_explorer.forms import MediaFormField, RichTextFormField

from django.db.models import signals, FileField
from django.forms import forms
from django.template.defaultfilters import filesizeformat


def parse_media(string_or_obj):
    """Takes a JSON string, converts it into a Media object."""
    data = {}
    kwargs = {}
    kwargs["id"] = None
    kwargs["type"] = None
    kwargs["caption"] = None
    kwargs["credit"] = None
    try:
        if type(string_or_obj) is dict:
            data = string_or_obj
        elif type(string_or_obj) is Element:
            data["id"] = string_or_obj.id
            data["type"] = string_or_obj.type
            data["caption"] = string_or_obj.description
            data["credit"] = string_or_obj.credit
        elif type(string_or_obj) is Gallery:
            data["id"] = string_or_obj.id
            data["type"] = "gallery"
        else:
            data = json.loads(string_or_obj)
    except Exception as e:
        raise ValidationError("Media parsing error: " + str(e))

    if data:
        kwargs.update(data)
    return Media(**kwargs)

def parse_richtext(text):
    """Takes a string, converts it into a RichText object."""
    return RichText(text)


class Media(object):
    """The corresponding Python object for the Django MediaField."""
    def __init__(self,id=None,type=None,caption=None,credit=None):
        self.id = id
        self.type = type
        self.caption = caption
        self.credit = credit

    def to_dict(self):
        _dict = {}
        _dict["id"] = self.id
        _dict["type"] = self.type
        _dict["caption"] = self.caption
        _dict["credit"] = self.credit
        return _dict

    def __repr__(self):
        #return "[MediaField object]: id=%s, type=%s" % (self.id, self.type)
        return json.dumps(self.to_dict())

class MediaField(models.TextField):
    """The Django MediaField."""

    description = _("A Media Explorer custom model field")

    def __init__(self, id=None, type=None, \
            credit=None, caption=None, *args, **kwargs):
        self.id = id
        self.type = type
        self.caption = caption
        self.credit = credit

        kwargs['null'] = True
        kwargs['blank'] = True

        super(MediaField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(MediaField, self).deconstruct()
        del kwargs["null"]
        del kwargs["blank"]
        return name, path, args, kwargs

    def db_type(self, connection):
        return "longtext"

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value
        return parse_media(value)

    def do_validation(self, media):
        if self.type and media.type and self.type != media.type:
            raise ValidationError("Invalid media type selected for this MediaField instance. It expected a '%s' but got a '%s' instead." % (self.type, media.type))

        #Override id/credit/caption
        if self.id : media.id = self.id
        if self.type: media.type = self.type
        if self.caption : media.caption = self.caption
        if self.credit: media.credit = self.credit

        #Validate that the image/video is in the system
        if media.type in ["image","video"] and \
                not Element.objects.filter(id=media.id,type=media.type).exists():
            raise ValidationError("Invalid %s selected. The %s was not found." % (media.type, media.type))

        #Validate that the gallery is in the system
        if media.type == "gallery" and \
                not Gallery.objects.filter(id=media.id).exists():
            raise ValidationError("Invalid %s selected. The %s was not found." % (media.type, media.type))

        return media

    def to_python(self, value):
        if isinstance(value, Media):
            return value

        if value is None:
            return value

        return self.do_validation(parse_media(value))
    
    def get_prep_value(self, value):
        value_dict = {}

        try:
            value_dict["id"] = value.id
            value_dict["type"] = value.type
            value_dict["caption"] = value.caption
            value_dict["credit"] = value.credit
        except:
            pass

        if type(value) is Element:
            value_dict["id"] = value.id
            value_dict["type"] = value.type
            value_dict["caption"] = None
            value_dict["credit"] = None
            if value.description:
                value_dict["caption"] = value.description
            if value.credit:
                value_dict["credit"] = value.credit

        if value_dict:
            self.do_validation(parse_media(value_dict))
            return str(json.dumps(value_dict))

        if value: return str(value)
        return value

    def formfield(self, **kwargs):
        defaults = {}
        defaults["form_class"] = MediaFormField
        defaults.update(kwargs)
        return super(MediaField, self).formfield(**defaults)


class RichText(object):
    """The corresponding Python object for the Django RichTextField."""

    def __init__(self,text):
        self.text = text

    def __repr__(self):
        return self.text

class RichTextField(models.TextField):
    """The Django RichTextField."""

    description = _("A RichText Explorer custom model field")

    def __init__(self, *args, **kwargs):
        kwargs['null'] = True
        kwargs['blank'] = True
        super(RichTextField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(RichTextField, self).deconstruct()
        del kwargs["null"]
        del kwargs["blank"]
        return name, path, args, kwargs

    def db_type(self, connection):
        return "longtext"

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value
        return parse_richtext(value)

    def do_validation(self, richtext):
        return richtext

    def to_python(self, value):
        if isinstance(value, RichText):
            return value

        if value is None:
            return value

        return self.do_validation(parse_richtext(value))
    

    def get_prep_value(self, value):
        if value: return str(value)
        return value

    def formfield(self, **kwargs):
        defaults = {}
        defaults["form_class"] = RichTextFormField
        defaults.update(kwargs)
        return super(RichTextField, self).formfield(**defaults)

class MediaImageField(FileField):
    """
    Forked from: https://djangosnippets.org/snippets/2206
    Same as FileField, but you can specify:
        * max_upload_size - a number indicating the maximum file size allowed for upload.
            2.5MB - 2621440
            5MB - 5242880
            10MB - 10485760
            20MB - 20971520
            50MB - 5242880
            100MB 104857600
            250MB - 214958080
            500MB - 429916160
    """
    def __init__(self, *args, **kwargs):
        self.content_types = []
        self.max_upload_size = 0
        self.new_upload = False

        #try:
        #    self.content_types = ["image/png","image/jpeg","image/jpg","image/bmp","image/gif","image/tiff","image/ief","image/g3fax"]
        #except Exception as e:
        #    pass

        try:
            self.max_upload_size = kwargs.pop("max_upload_size")
        except Exception as e:
            pass

        super(MediaImageField, self).__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        data = super(MediaImageField, self).clean(*args, **kwargs)

        file = data.file
        #content_type = file.content_type
        content_type = getattr(file,"content_type",None)

        if "django.core.files.uploadedfile.InMemoryUploadedFile" in str(type(file)):
            self.new_upload = True

        if content_type and not content_type.lower().startswith("image/"):
            raise forms.ValidationError(_('The file you selected is not an image. Please select an image.'))

        if self.max_upload_size > 0 and \
                file._size > self.max_upload_size:
            raise forms.ValidationError(_('Please keep filesize under %s. Current filesize %s') % (filesizeformat(self.max_upload_size), filesizeformat(file._size)))

        return data

    def contribute_to_class(self, cls, name, **kwargs):
        super(MediaImageField, self).contribute_to_class( cls, name, **kwargs)
        signals.post_save.connect(self.on_post_save_callback, sender=cls)
        #signals.post_delete.connect(self.on_post_delete_callback, sender=cls)

    def on_post_save_callback(self, instance, force=False, *args, **kwargs):
        """
        Save image into Element model
        """
        if self.new_upload and \
                type(instance.__dict__[self.name]) in [str,unicode]:
            data = {}
            data["image"] = instance.__dict__[self.name]
            element = Element()
            element.__dict__.update(data)
            element.save()

    #def on_post_delete_callback(self, instance, force=False, *args, **kwargs):
    #    """
    #    TODO
    #    Delete file from Element model
    #    """
    #    pass



from __future__ import unicode_literals
import os
from django.contrib.auth.models import User
from django.db import connection
from django.core.urlresolvers import reverse
from django.test import TestCase, Client, modify_settings, override_settings
from django.core.management import call_command 
from media_explorer.models import Element, Gallery
from media_explorer.fields import MediaField, RichTextField
from django.core.exceptions import ValidationError

@modify_settings(INSTALLED_APPS={
    'append': 'media_explorer.tests.customfields',
})
class CustomFieldTests(TestCase):

    def setUp(self):
        #Create test users
        User.objects.create_user("testuser",password="password")
        User.objects.create_superuser("admin","admin@example.com","password")

    def test_media_field_type(self):
        f = MediaField()
        self.assertEqual(f.db_parameters(connection)['type'], 'text')

    @override_settings(DME_RESIZE=False)
    def test_mediafield_assign_image_to_video_field(self):
        """
        Test image assignment to MediaField with type=video
        Condition: Authorized user tries to assign image to video field
        Result: Model returns a validation error
        """
        from media_explorer.tests.customfields.models import Example

        #Upload an image
        url = reverse("api-media-elements")
        c = Client()
        c.login(username="admin",password="password")

        test_path = os.path.dirname(os.path.abspath(__file__))
        with open(test_path + '/../elements/Oxfam-Cambodia.jpg') as fp:
            response = c.post(url, {'name':'test_mediafield_assign_image_to_video_field','image':fp},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        #Get the image we just uploaded
        image = Element.objects.get(type="image",name="test_mediafield_assign_image_to_video_field",image_url__icontains="Oxfam-Cambodia")

        #Now create Example object and assign image to video
        #NOTE: Do not save because the table is not created in test db
        error_message = ""
        example = Example()
        example.video = image
        try:
            example.clean_fields()
        except ValidationError as e:
            error_message = e.message_dict["video"][0]

        self.assertTrue("Invalid media type" in error_message)


    def test_mediafield_assign_video_to_image_field(self):
        """
        Test video assignment to MediaField with type=image
        Condition: Authorized user tries to assign video to image field
        Result: Model returns a validation error
        """
        from media_explorer.tests.customfields.models import Example

        #Save a video
        url = reverse("api-media-elements")

        c = Client()
        c.login(username="admin",password="password")

        response = c.post(url, {'name':'test_mediafield_assign_image_to_video_field','video_url':'https://www.youtube.com/watch?v=nBupSKjSQM0'},HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(response.status_code, 200)

        #Check the DB to make sure element is present
        video = Element.objects.get(type="video",name="test_mediafield_assign_image_to_video_field",video_embed__isnull=False)

        #Now create Example object and assign video to image
        #NOTE: Do not save because the table is not created in test db
        error_message = ""
        example = Example()
        example.image = video
        try:
            example.clean_fields()
        except ValidationError as e:
            error_message = e.message_dict["image"][0]

        self.assertTrue("Invalid media type" in error_message)




import os

# Comment this if you have already set it
PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

# Comment this if you have already set it
STATIC_URL = "/static/"

# Comment this if you have already set it
STATIC_ROOT = os.path.join(PROJECT_ROOT, STATIC_URL.strip("/"))

# Comment this if you have already set it
MEDIA_URL = STATIC_URL + "media/"

# Comment this if you have already set it
MEDIA_ROOT = os.path.join(PROJECT_ROOT, *MEDIA_URL.strip("/").split("/"))

DME_RESIZE = True

DME_VIDEO_THUMBNAIL_DEFAULT_URL = "/static/img/default_video.gif"
DME_GALLERY_THUMBNAIL_DEFAULT_URL = "/static/img/default_gallery.gif"

#This will be appended to settings.MEDIA_URL
DME_RESIZE_DIRECTORY = "resized"

DME_RESIZE_HORIZONTAL_ASPECT_RATIO = "8:5"
DME_RESIZE_VERTICAL_ASPECT_RATIO = "320:414"

DME_RESIZE_WIDTHS = {
    "horizontal": [2440,1220,840,800,610,420,160],
    "vertical": [556,320,278,160],
    "non_cropped": [2440,1220,610,160],
    "retina_2x": [1220,610,420,278,160],
    "thumbnail": 200,
}

DME_PAGE_SIZE = 50

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
    ],
    'DATETIME_FORMAT': "%Y-%m-%d %T",
}

CKEDITOR_JQUERY_URL = "http://code.jquery.com/jquery-1.11.2.min.js"

CKEDITOR_UPLOAD_PATH = "uploads/"

CKEDITOR_CONFIGS = {
    'default': {
        'extraPlugins': 'media_explorer',
        'toolbar': 'Custom',
        'toolbar_Custom': [
            ['Bold', 'Italic', 'Underline'],
            ['NumberedList', 'BulletedList', '-', 'Outdent', 'Indent', '-', 'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock'],
            ['Link', 'Unlink'],
            ['RemoveFormat', 'Source'],
            ['MediaExplorer']
        ]
    }
}


